/* file - mlisten.c                                                         */
/*                                                                          */
/* Version:  1.0                                                            */
/* Written:  20-Jun-95                                                      */
/* Release:  10-Oct-95                                                      */
/*                                                                          */
/* Kevin Almeroth (kevin@cc.gatech.edu)                                     */
/* Networking and Telecommunications Group                                  */
/* College of Computing, Georgia Institute of Technology                    */
/* http://www.cc.gatech.edu/computing/Telecomm/people/Phd/kevin/kevin.html  */

/*
  Copyright 1995
  Georgia Tech Research Corporation
  Atlanta, Georgia  30332-0415
  ALL RIGHTS RESERVED

  This material may be modified, copied and redistributed,
  both within the recipient's organization and externally, subject to the
  following restrictions:  (a)  The recipient may not derive income
  from the Georgia Tech Research Corporation (herein "GTRC")  information
  itself;  (b)  In any material based on this information, the recipient
  agrees to acknowledge GTRC; (c)  Any copies made of this material must be
  accompanied by the following copyright notice: "Copyright 1995 Georgia
  Tech Research Corporation.  Atlanta, GA  30332-0145.  All rights Reserved.";
   and (d)  The recipient agrees to obey all U.S. Government restrictions
  governing redistribution or export of such information.  These restrictions
  may apply to redistribution within an international organization.  GTRC
  makes no warranties or representations, either expressed or implied, with
  respect to the results containied herein, its quality, merchantability,
  performance or fitness for a particular purpose.  In no event shall GTRC or
  its developers, directors, officers, employees or affiliates be liable for
  direct, incidental, indirect, special or consequential damages (including
  damages or loss of business profits, business interruption, loss of business
  information and the like) resulting from any defect in this material or its
  documentation or arising out of the use or inability to use this material
  or accompanying documentation even if GTRC, an authorized representative or
  a GTRC affiliate has been advised of the possibility of such damage.  GTRC
  makes no representation or warranty regarding the results obtainable through
  use of this material.  No oral or written information or advice given by
  GTRC, its dealers, distributors, agents, affiliates, developers, directors,
  officers or employees shall create a warranty or in any way increase the
  scope of this warranty.
*/

#include <stdio.h>
#include <sys/types.h>

#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>


#define ACTIVE 0
#define FINISHED 1

#define FILE_NAME_SIZE 80
#define DATA_SIZE 40

char IP_Multicast_Address[DATA_SIZE];
char UDP_Port_Number[DATA_SIZE];
int  Connection_End_Threshold;
int  Cycle_Length;
int  Write_Frequency;
char Completed_Connections_File_Name[FILE_NAME_SIZE];
char Active_Connections_File_Name[FILE_NAME_SIZE];

typedef struct
{
   char name[DATA_SIZE];
   int  start_day_num;
   char start_date[10];
   char start_time[10];
   int  latest_day_num;
   char latest_date[10];
   char latest_time[10];
   char end_date[10];
   char end_time[10];
   int  received;
   int  status;
   void *next_ptr;
} info;

info *Head_Pointer = NULL;
info *addr_ptr = NULL;

int listeners;
time_t *CLOCK;
struct tm *local_time;


FILE *out_file;
int  socket_id;

int  latest_check_day_num;
char latest_check_date[10];
char latest_check_time[10];

int  current_day_num;
char current_date[10];
char current_time[10];

int checks = 0;

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Read_Parameters()
{
   FILE *param_file;

   param_file = fopen("params","r");

   if (param_file == NULL)
   {
      printf("Could not find params file in the current directory.\n");
      exit(-1);
   }

   fscanf(param_file,"%*s = %s\n",IP_Multicast_Address);
   fscanf(param_file,"%*s = %s\n",UDP_Port_Number);
   fscanf(param_file,"%*s = %d\n",&Connection_End_Threshold);
   fscanf(param_file,"%*s = %d\n",&Cycle_Length);
   fscanf(param_file,"%*s = %d\n",&Write_Frequency);
   fscanf(param_file,"%*s = %s\n",Completed_Connections_File_Name );
   fscanf(param_file,"%*s = %s\n",Active_Connections_File_Name);

/*
printf("IP Multicast Address is %s.\n",IP_Multicast_Address);
printf("UDP Port Number is %s.\n",UDP_Port_Number);
printf("Connection End Threshold is %d.\n",Connection_End_Threshold);
printf("Cycle Length is %d.\n",Cycle_Length);
printf("Write Frequency is %d.\n",Write_Frequency);
*/

   fclose(param_file);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
info *Search_Records(passed_addr)
char *passed_addr;
{
   info *scan_ptr = Head_Pointer;

   while (scan_ptr != NULL)
   {
      if (strcmp(passed_addr,scan_ptr->name) == 0)
      {
         if (scan_ptr->status == ACTIVE)
            return(scan_ptr);
      }

      scan_ptr = scan_ptr->next_ptr;
   }
   return(NULL);
}
/*************************************************************************/


/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Add_Record(passed_addr)
char *passed_addr;
{
   info *scan_ptr = Head_Pointer;
   info *new_ptr;

   if (scan_ptr != NULL)
   {
      while (scan_ptr->next_ptr != NULL)
      {
         scan_ptr = scan_ptr->next_ptr;
      }
   }

   new_ptr = (info*)malloc(sizeof(info));
   
   strcpy(new_ptr->name,passed_addr);
   new_ptr->status = ACTIVE;
   new_ptr->received = 1;
   new_ptr->next_ptr = NULL;
   new_ptr->start_day_num = Get_Date_Offset();
   Get_Date(new_ptr->start_date);
   Get_Time(new_ptr->start_time);
   new_ptr->latest_day_num = new_ptr->start_day_num;
   strncpy(new_ptr->latest_date,new_ptr->start_date,10);
   strncpy(new_ptr->latest_time,new_ptr->start_time,10);

   if (Head_Pointer == NULL)
      Head_Pointer = new_ptr;
   else
      scan_ptr->next_ptr = new_ptr;

   listeners++;
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Get_Time(request_var)
char request_var[10];
{
   time(CLOCK);
   local_time = localtime(CLOCK);
   strftime(request_var, 10, "%T", local_time);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Get_Date(request_var)
char request_var[10];
{
   time(CLOCK);
   local_time = localtime(CLOCK);
   strftime(request_var, 10, "%D", local_time);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
int Get_Date_Offset()
{
   char temp[10];
   int value;

   time(CLOCK);
   local_time = localtime(CLOCK);
   strftime(temp, 10, "%j", local_time);
   value = atoi(temp);
   return(value);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Check_For_Departures()
{
   info *scan_ptr = Head_Pointer;
   int diff;

   while (scan_ptr != NULL)
   {
      if (scan_ptr->status == ACTIVE)
      {
         diff = Time_Difference(current_day_num,current_time,
                                scan_ptr->latest_day_num,
                                scan_ptr->latest_time);
         if (diff > Connection_End_Threshold)
         {
            scan_ptr->status = FINISHED;
            strcpy(scan_ptr->end_date,current_date);
            strcpy(scan_ptr->end_time,current_time);
         }
      }
      scan_ptr = scan_ptr->next_ptr;
   }
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
int Time_Difference(new_day_num,new_time,old_day_num,old_time)
int  new_day_num;
char new_time[10];
int  old_day_num;
char old_time[10];
{
int hour;
int minute;
int second;
int result;

   result = (new_day_num - old_day_num) * (60*60*24);

   sscanf(new_time,"%2d:%2d:%2d",&hour,&minute,&second);
   result += ((hour * 3600) + (minute * 60) + second);

   sscanf(old_time,"%2d:%2d:%2d",&hour,&minute,&second);
   result -= ((hour * 3600) + (minute * 60) + second);

   return(result);

}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Print_Results(type)
int type;
{
   info *scan_ptr = Head_Pointer;

   while(scan_ptr != NULL)
   {

      if (scan_ptr->status == type)
      {
         printf("------------------------------------");
         printf("------------------------------------\n");
         printf("Name:  `%s'.  ",scan_ptr->name);
         printf("Received:  %d.  ",scan_ptr->received);
         printf("Status:  %d.\n",scan_ptr->status);

         printf("Start Date:  %s.  Start Time:  %s.\n",
                scan_ptr->start_date, scan_ptr->start_time);

         printf("Latest Date: %s.  Latest Time: %s.\n",
                scan_ptr->latest_date, scan_ptr->latest_time);

         printf("End Date:    %s.  End Time:    %s.\n",
                scan_ptr->end_date, scan_ptr->end_time);
         printf("------------------------------------");
         printf("------------------------------------\n");
      }

      scan_ptr = scan_ptr->next_ptr;
   }
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Write_Finished_Records()
{
   info *scan_ptr1 = Head_Pointer;
   info *scan_ptr2 = Head_Pointer;

   out_file = fopen(Completed_Connections_File_Name,"a");

   if (out_file == NULL)
   {
      printf("Cannot open file %s to write completed connection list.\n",
             Completed_Connections_File_Name);
      exit(-1);
   }

   while(scan_ptr2 != NULL)
   {
      if (scan_ptr2->status == FINISHED)
      {
         fprintf(out_file,"------------------------------------");
         fprintf(out_file,"------------------------------------\n");
         fprintf(out_file,"Name:  `%s'.  ",scan_ptr2->name);
         fprintf(out_file,"Received:  %d.  ",scan_ptr2->received);
         fprintf(out_file,"Status:  %d.\n",scan_ptr2->status);

         fprintf(out_file,"Start Date:  %s.  Start Time:  %s.\n",
                 scan_ptr2->start_date, scan_ptr2->start_time);

         fprintf(out_file,"Latest Date: %s.  Latest Time: %s.\n",
                 scan_ptr2->latest_date, scan_ptr2->latest_time);

         fprintf(out_file,"End Date:    %s.  End Time:    %s.\n",
                 scan_ptr2->end_date, scan_ptr2->end_time);
         fprintf(out_file,"------------------------------------");
         fprintf(out_file,"------------------------------------\n");

         if (scan_ptr2 == Head_Pointer)
         {
            Head_Pointer = scan_ptr2->next_ptr;
            free(scan_ptr2);
            scan_ptr1 = Head_Pointer;
            scan_ptr2 = Head_Pointer;
         }
         else
         {
            scan_ptr1->next_ptr = scan_ptr2->next_ptr;
            free(scan_ptr2);
            scan_ptr2 = scan_ptr1->next_ptr;
         }

         listeners--;
      }
      else
      {
         scan_ptr1 = scan_ptr2;
         scan_ptr2 = scan_ptr2->next_ptr;
      }
   }
   fclose(out_file);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Write_Active_Records()
{
   info *scan_ptr = Head_Pointer;

   out_file = fopen(Active_Connections_File_Name,"w");

   if (out_file == NULL)
   {
      printf("Cannot open file %s to write active connection list.\n",
             Active_Connections_File_Name);
      exit(-1);
   }

   while(scan_ptr != NULL)
   {
      if (scan_ptr->status == ACTIVE)
      {
         fprintf(out_file,"------------------------------------");
         fprintf(out_file,"------------------------------------\n");
         fprintf(out_file,"Name:  `%s'.  ",scan_ptr->name);
         fprintf(out_file,"Received:  %d.  ",scan_ptr->received);
         fprintf(out_file,"Status:  %d.\n",scan_ptr->status);

         fprintf(out_file,"Start Date:  %s.  Start Time:  %s.\n",
                 scan_ptr->start_date, scan_ptr->start_time);

         fprintf(out_file,"Latest Date: %s.  Latest Time: %s.\n",
                 scan_ptr->latest_date, scan_ptr->latest_time);

         fprintf(out_file,"End Date:    %s.  End Time:    %s.\n",
                 scan_ptr->end_date, scan_ptr->end_time);

         fprintf(out_file,"------------------------------------");
         fprintf(out_file,"------------------------------------\n");
      }

      scan_ptr = scan_ptr->next_ptr;
   }
   fclose(out_file);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
int Open_Socket()
{
   int                sockfd;
   struct ip_mreq     mreq;
   struct sockaddr_in serv_addr;

   if ( (sockfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
      printf("server: can't open datagram socket");

   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family      = AF_INET;
   serv_addr.sin_addr.s_addr = inet_addr(IP_Multicast_Address);
   serv_addr.sin_port        = htons(atoi(UDP_Port_Number));

   if (bind(sockfd, (struct sockaddr*) &serv_addr, sizeof(serv_addr)) < 0)
      printf("server: can't bind local address");

   mreq.imr_multiaddr = serv_addr.sin_addr;
   mreq.imr_interface.s_addr = INADDR_ANY;

   if (setsockopt(sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                  &mreq, sizeof(mreq)) == -1)
   {
      printf("server: setsockopt failed \n");
      exit (1);
   }

   return(sockfd);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
main()
{
   int rc;
   int time_diff;
   char *address;
   char *port;
   char *packet_data;
   struct sockaddr *socket_addr;
   struct sockaddr_in *socket_addr_id;
   int    *len;
   char   message;


   Read_Parameters();

   socket_id = Open_Socket();

   CLOCK = (time_t*)malloc(sizeof(time_t));
   local_time = (struct tm*)malloc(sizeof(struct tm));
   packet_data = (char*)malloc(80);
   socket_addr = (struct sockaddr *)malloc(sizeof(struct sockaddr));
   socket_addr_id = (struct sockaddr_in*)socket_addr;
   len = (int*)malloc(sizeof(int));

   address = (char*)malloc(DATA_SIZE);
   port = (char*)malloc(DATA_SIZE);

   latest_check_day_num = Get_Date_Offset();
   current_day_num = Get_Date_Offset();
   Get_Date(latest_check_date);
   Get_Date(current_date);
   Get_Time(latest_check_time);
   Get_Time(current_time);

   out_file = fopen(Completed_Connections_File_Name,"w");

   if (out_file == NULL)
   {
      printf("Cannot open file %s to write completed connection list.\n",
             Completed_Connections_File_Name);
      exit(-1);
   }

   fprintf(out_file,"Start time:  %s(%s).\n",
           latest_check_date,latest_check_time);
   fprintf(out_file,"Multicast Address:  %s.  Multicast Port:  %s.\n",
           IP_Multicast_Address,UDP_Port_Number);
   fclose(out_file);

   Do_Check();

   while(1)
   {
      if (recvfrom(socket_id, &message, 1, 0, socket_addr, &len) < 0)
         printf("recvfrom error\n");
      else
      {
         strcpy(address,inet_ntoa(socket_addr_id->sin_addr));
         strcat(address,".");
         sprintf(port,"%d",ntohs(socket_addr_id->sin_port));
         strcat(address,port);
      
         addr_ptr = Search_Records(address);

         /* No matching record was found */
         if (addr_ptr == NULL)
         {
            Add_Record(address);
         }
         else
         {
            addr_ptr->received++;
            addr_ptr->latest_day_num = Get_Date_Offset();
            Get_Date(addr_ptr->latest_date);
            Get_Time(addr_ptr->latest_time);
         }
      }
     
      current_day_num = Get_Date_Offset();
      Get_Date(current_date);
      Get_Time(current_time);

      time_diff = Time_Difference(current_day_num,current_time,
                                  latest_check_day_num,latest_check_time);

      if (time_diff >= Cycle_Length)
         Do_Check();
   }
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Do_Check()
{

   Check_For_Departures();
   Print_Results(FINISHED);
   Write_Finished_Records();

   printf("%s(%s):  Listeners %3d.  ",current_date,current_time,listeners);

   latest_check_day_num = current_day_num;
   strcpy(latest_check_date,current_date);
   strcpy(latest_check_time,current_time);

   checks++;

   if (checks >= Write_Frequency)
   {
      printf("Writing active records to file...\n");
      Write_Active_Records();
      checks = 0;
   }
   else
      printf("\n");
}
/*************************************************************************/
