/* file - mprocess.c                                                        */
/*                                                                          */
/* Version:  1.0                                                            */
/* Written:  20-Jun-95                                                      */
/* Release:  10-Oct-95                                                      */
/*                                                                          */
/* Kevin Almeroth (kevin@cc.gatech.edu)                                     */
/* Networking and Telecommunications Group                                  */
/* College of Computing, Georgia Institute of Technology                    */
/* http://www.cc.gatech.edu/computing/Telecomm/people/Phd/kevin/kevin.html  */

/*
  Copyright 1995
  Georgia Tech Research Corporation
  Atlanta, Georgia  30332-0415
  ALL RIGHTS RESERVED

  This material may be modified, copied and redistributed,
  both within the recipient's organization and externally, subject to the
  following restrictions:  (a)  The recipient may not derive income
  from the Georgia Tech Research Corporation (herein "GTRC")  information
  itself;  (b)  In any material based on this information, the recipient
  agrees to acknowledge GTRC; (c)  Any copies made of this material must be
  accompanied by the following copyright notice: "Copyright 1995 Georgia
  Tech Research Corporation.  Atlanta, GA  30332-0145.  All rights Reserved.";
   and (d)  The recipient agrees to obey all U.S. Government restrictions
  governing redistribution or export of such information.  These restrictions
  may apply to redistribution within an international organization.  GTRC
  makes no warranties or representations, either expressed or implied, with
  respect to the results containied herein, its quality, merchantability,
  performance or fitness for a particular purpose.  In no event shall GTRC or
  its developers, directors, officers, employees or affiliates be liable for
  direct, incidental, indirect, special or consequential damages (including
  damages or loss of business profits, business interruption, loss of business
  information and the like) resulting from any defect in this material or its
  documentation or arising out of the use or inability to use this material
  or accompanying documentation even if GTRC, an authorized representative or
  a GTRC affiliate has been advised of the possibility of such damage.  GTRC
  makes no representation or warranty regarding the results obtainable through
  use of this material.  No oral or written information or advice given by
  GTRC, its dealers, distributors, agents, affiliates, developers, directors,
  officers or employees shall create a warranty or in any way increase the
  scope of this warranty.
*/


#include <stdio.h>
#include <sys/stdtypes.h>
#include <sys/time.h>

#define START 0
#define STOP 1500000

#define NUM_RECORDS (STOP - START + 1000)
#define MAX_NUM_NAMES 10000
#define NAME_LENGTH 22

FILE *fd1;
FILE *fd2;

struct tm *time_struct;

time_t *current_time;
time_t *records_start;
time_t *records_stop;
time_t *start_secs;

FILE *out_file;
char file[40];

char conf_start_date[10];
char conf_start_time[10];

char records_name[NAME_LENGTH];
char date[10];
char hour[10];

typedef struct
{
   int listeners;
   int join;
   int leave;
} stat_record;

stat_record stats[NUM_RECORDS];

typedef struct
{
   int start;
   int stop;
   char *name;
} duration_record;

duration_record duration[MAX_NUM_NAMES+1];

int free_record = 1;

int CHECK_INCREMENT;
int loop;
int records_processed;
int total_listeners;
int total_join;
int total_leave;
int print_record=0;
int total_check=0;

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
time_t String_To_Secs(date,hour)
char date[10];
char hour[10];
{
   strptime(date,"%D",time_struct);
   strptime(hour,"%T",time_struct);
   return(timelocal(time_struct));
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Time_To_Hour(secs)
time_t secs;
{
   time_struct = localtime(secs); 
   strftime(hour,10,"%T",time_struct);
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Time_To_Date(secs)
time_t secs;
{
   time_struct = localtime(secs); 
   strftime(date,10,"%D",time_struct);
}
/*************************************************************************/



/*************************************************************************/
/*                                                                       */
/*************************************************************************/
int Time_Diff(new_date,new_time,old_date,old_time)
char new_date[10];
char new_time[10];
char old_date[10];
char old_time[10];
{
int hour;
int minute;
int second;
int result;

int new_day;
int new_month;
int new_year;

int old_day;
int old_month;
int old_year;
   
   sscanf(new_date,"%2d/%2d/%2d",&new_month,&new_day,&new_year);
   sscanf(old_date,"%2d/%2d/%2d",&old_month,&old_day,&old_year);

   if ((new_year != old_year) || (new_month != old_month))
   {
      printf("ERROR:  Crossed year or month boundary.\n");
      printf("New year: %d.  Old year: %d.  New month: %d.  Old month %d.\n",
             new_year,old_year,new_month,old_month);
      exit(-1);
   }

   result = (new_day - old_day) * (60*60*24);

   sscanf(new_time,"%2d:%2d:%2d",&hour,&minute,&second);
   result += ((hour * 3600) + (minute * 60) + second);

   sscanf(old_time,"%2d:%2d:%2d",&hour,&minute,&second);
   result -= ((hour * 3600) + (minute * 60) + second);

   return(result);

}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Read_Record()
{
   if ((fscanf(out_file,"%*[^\n]\n")) == -1)
      return(-1);

   fscanf(out_file,"Name:  `%[0-9,.]",records_name);
   fscanf(out_file,"%*[^\n]\n");


   fscanf(out_file,"Start Date:  %[0-9,/].  Start Time:  %[0-9,:].\n",
          date,hour);
   *records_start = String_To_Secs(date,hour);

   fscanf(out_file,"Latest Date:  %[0-9,/].  Latest Time:  %[0-9,:].\n",
          date,hour);
   *records_stop = String_To_Secs(date,hour);

   fscanf(out_file,"%*[^\n]\n");
   fscanf(out_file,"%*[^\n]\n");

   return(0);
}

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Read_Header()
{
   fscanf(out_file,"Start time:  %8s(%8s).\n",conf_start_date,conf_start_time);
   fscanf(out_file,"%*[^-]\n");
}
/*************************************************************************/


/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Check_Membership()
{

/*
   printf("Name:  %s.\n",records_name);
*/

   for(loop=1; loop<free_record; loop++)
   {
      if (strcmp(duration[loop].name,records_name) == 0)
      {
         /* If the record has already has an entry, adjust end time */
/*
         printf("SAME RECORD(%d):  %s.  Modifying end time...\n",
                loop,duration[loop].name);
*/
         duration[loop].stop = (int)*records_stop;
         return;
      }
   }
   strncpy(duration[free_record].name,records_name,NAME_LENGTH);
   duration[free_record].start = (int)*records_start;
   duration[free_record].stop = (int)*records_stop;
/*
   printf("ADDING RECORD(%d):  %s.  Start:  %d.  Stop %d.\n",
          free_record,duration[free_record].name,
          duration[free_record].start,
          duration[free_record].stop);
*/
   free_record++;
}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
Process_Record(pointer)
int pointer;
{
   int stat_start;
   int stat_stop;
   int loopa;


   stat_start = duration[pointer].start - (int)*start_secs;
   stat_stop  = duration[pointer].stop - (int)*start_secs;

/*
   printf("Start is %d.  Stop is %d.\n",stat_start,stat_stop);
*/

   if (stat_stop < START)
      return;
   if (stat_start > STOP)
      return;

      /* Need to do this so stops are arbitrarily inflated */
   if (stat_stop > STOP)
   {
      stat_stop = STOP;
   }
   else
   {
      stats[stat_stop-START].leave++;
   }

   if (stat_start < START)
   {
      stat_start = START;
   }
   else
   {
      stats[stat_start-START].join++;
   }

   for(loopa=stat_start; loopa<=stat_stop; loopa++)
   {
      stats[loopa-START].listeners++;
   }

}
/*************************************************************************/

/*************************************************************************/
/*                                                                       */
/*************************************************************************/
main(argc,argv)
int argc;
char *argv[];
{
   int max=0;

   if (argc > 1)
      sscanf(argv[1],"%s",file);
   else
      strcpy(file,"data");

   if (argc > 2)
      sscanf(argv[2],"%d",&CHECK_INCREMENT);
   else
      CHECK_INCREMENT = 300;

   
   time_struct = (struct tm*)malloc(sizeof(struct tm));

   start_secs = (time_t*)malloc(sizeof(time_t));
   current_time = (time_t*)malloc(sizeof(time_t));
   records_start = (time_t*)malloc(sizeof(time_t));
   records_stop = (time_t*)malloc(sizeof(time_t));

   for(loop=0; loop<=MAX_NUM_NAMES; loop++)
   {
      duration[loop].name = (char*)malloc(NAME_LENGTH);
   }

   for(loop=0; loop<NUM_RECORDS; loop++)
   {
      stats[loop].listeners = 0;
      stats[loop].join = 0;
      stats[loop].leave = 0;
   }

   records_processed = 0;

   out_file = fopen(file,"r");

   if (out_file == NULL)
   {
      printf("Cannot open file `%s' for reading.\n",file);
      printf("Usage: mprocess <in_file> <increment_size>\n");
      exit(-1);
   }

   Read_Header();

   printf("Conference Start Date:  %s.  Conference Start Time:  %s.\n",
          conf_start_date,conf_start_time);

   *start_secs   = String_To_Secs(conf_start_date,conf_start_time);
   *current_time = String_To_Secs(conf_start_date,conf_start_time);

   printf("Reading Records From File...\n\n");
   while (Read_Record() == 0)
   {
      Check_Membership();
      records_processed++;
   }

   fclose(out_file);

   printf("Processing Records...\n\n");
   for(loop=1; loop<free_record; loop++)
   {
      Process_Record(loop);
   }
   
   printf("Processed %d record(s) and %d unique connections.\n",
          records_processed,(free_record-1));

   strcat(file,"-out");

   fd1 = fopen(file,"w");

   /* Find the last record */
   for(loop=0; loop<NUM_RECORDS; loop++)
   {
      if ((stats[loop].listeners != 0) || (stats[loop].join != 0) ||
          (stats[loop].leave != 0))
      {
         print_record = loop;
      }
      if (stats[loop].listeners > max)
      {
         max = stats[loop].listeners;
      } 
   }
   print_record += (START + CHECK_INCREMENT);

   if (STOP < print_record)
      print_record = STOP;

   printf("Maximum number of listeners at any one time is %d.\n\n",max);
   printf("Writing Output Data File...\n\n");

   total_join = 0;
   total_leave = 0;
   total_check = 0;
   for(loop=START; loop<=print_record; loop++)
   {
      total_listeners += stats[loop-START].listeners;
      total_join += stats[loop-START].join;
      total_leave += stats[loop-START].leave;
      total_check += (stats[loop-START].join - stats[loop-START].leave);
      if ((loop % CHECK_INCREMENT) == 0)
      {
         fprintf(fd1,"%7d%7.2f%5d%5d%5d\n", loop,
                 ((float)total_listeners / (float)CHECK_INCREMENT),
                 (total_join - total_leave),
                 total_join,total_leave);
         total_listeners = 0;
         total_join = 0;
         total_leave = 0;
      }
   }
   fclose(fd1);
   printf("Done!\n");
}
/*************************************************************************/
